/*
 * Copyright 2000-2013 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.xmlnamespace.panel.vaadin.shared.permissions;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.vaadin.shared.AbstractComponentState;

/**
 *
 * @since
 * @author Vaadin Ltd
 */
public class PermissionsEditorState extends AbstractComponentState {

    private static final long serialVersionUID = -2093136170217568179L;

    public String titleHeader;

    public String policyHeader;

    public String typeHeader;

    public String exceptionsHeader;

    public boolean allowExceptions;

    public boolean allowInherit;

    public String policyInheritText;

    public String policyAllowText;

    public String policyDenyText;

    public String policyOwnedText;

    public String selectText;

    public String selectAllText;

    public String selectNoneText;

    public String setSelectedToText;

    public String exceptText;

    public String suggestionsText;

    public List<ClientPermissionGroup> groups = new ArrayList<ClientPermissionGroup>();

    public String subjectId;

    /**
     * An implementation detail - don't use this.
     *
     * @since 3.1
     * @author Colin Williams
     */
    public static class ClientPermissionRow implements Serializable,
            Comparable<ClientPermissionRow> {
        private static final long serialVersionUID = -8645124774093534811L;
        public String title;
        public int sortOrder;
        public String identifier;

        public boolean checked;

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Comparable#compareTo(java.lang.Object)
         */
        @Override
        public int compareTo(ClientPermissionRow o) {
            return sortOrder - o.sortOrder;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#hashCode()
         */
        @Override
        public int hashCode() {
            final int prime = 31;
            int result = 1;
            result = prime * result
                    + ((identifier == null) ? 0 : identifier.hashCode());
            return result;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#equals(java.lang.Object)
         */
        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ClientPermissionRow other = (ClientPermissionRow) obj;
            if (identifier == null) {
                if (other.identifier != null) {
                    return false;
                }
            } else if (!identifier.equals(other.identifier)) {
                return false;
            }
            return true;
        }
    }

    public static class ClientPermissionGroup extends ClientPermissionRow {
        private static final long serialVersionUID = 4319333261254822989L;
        public List<ClientPermissionType> types = new ArrayList<ClientPermissionType>();

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#equals(java.lang.Object)
         */
        @Override
        public boolean equals(Object obj) {
            if (obj instanceof ClientPermissionGroup && identifier != null) {
                return identifier
                        .equals(((ClientPermissionGroup) obj).identifier);
            }

            return false;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#toString()
         */
        @Override
        public String toString() {
            return "ClientPermissionGroup [title=" + title + "]";
        }
    }

    public static class ClientPermissionType extends ClientPermissionRow {
        private static final long serialVersionUID = 4493018982823556246L;

        /**
         * The default policy for this permission type. Values are "inherit",
         * "allow", or "deny"
         */
        public String policy;
        public String type;
        public List<String> exceptions;
        public Map<String, String> exceptionOptions;

        /**
         * If the policy is "inherit", this contains the policy that's being
         * inherited, for the purposes of displaying it to the user for clarity.
         * Valid values are "allow" or "deny".
         */
        public String inheritedPolicy;

        public boolean allowExceptions;
        public boolean allowEditExceptions;

        @Override
        public String toString() {
            return "ClientPermissionType [policy=" + policy + ", type=" + type
                    + ", exceptions=" + exceptions + ", title=" + title
                    + ", identifier=" + identifier + "]";
        }

    }
}
