package com.xmlnamespace.panel.vaadin.ui;

import com.vaadin.data.util.converter.Converter.ConversionException;
import com.vaadin.shared.MouseEventDetails;
import com.vaadin.ui.AbstractField;
import com.xmlnamespace.panel.vaadin.shared.ui.CustomCheckBoxServerRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.CustomCheckBoxState;

/**
 * CustomCheckBox Widget.
 */
public class CustomCheckBox extends AbstractField<Boolean> {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -1005042694836016247L;

    /** The rpc. */
    private CustomCheckBoxServerRpc rpc = new CustomCheckBoxServerRpc() {
        private static final long serialVersionUID = 5037135623080784195L;

        @Override
        public void setState(boolean checked, boolean indeterminate,
                MouseEventDetails mouseEventDetails) {
            if (isReadOnly()) {
                return;
            }

            getUI().getConnectorTracker().getDiffState(CustomCheckBox.this)
                    .put("checked", checked);
            getUI().getConnectorTracker().getDiffState(CustomCheckBox.this)
                    .put("indeterminate", indeterminate);

            final Boolean oldValue = getValue();
            final Boolean newValue = checked;

            final Boolean oldIndeterminate = isIndeterminate();
            final Boolean newIndeterminate = indeterminate;

            if (!newIndeterminate.equals(oldIndeterminate)) {
                setIndeterminate(newIndeterminate);
            }

            if (!newValue.equals(oldValue)) {
                setValue(newValue);
            }
        }
    };

    /**
     * Instantiates a new custom check box.
     */
    public CustomCheckBox() {
        registerRpc(rpc);
        setValue(Boolean.FALSE);
        setIndeterminate(Boolean.FALSE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractField#getType()
     */
    @Override
    public Class<? extends Boolean> getType() {
        return Boolean.class;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.vaadin.ui.AbstractField#getState()
     */
    @Override
    protected CustomCheckBoxState getState() {
        return (CustomCheckBoxState) super.getState();
    }

    /**
     * Checks if is indeterminate.
     * 
     * @return true, if is indeterminate
     */
    public boolean isIndeterminate() {
        return getState().indeterminate;
    }

    /**
     * Sets the indeterminate.
     * 
     * @param indeterminate
     *            the new indeterminate
     */
    public void setIndeterminate(boolean indeterminate) {
        getState().indeterminate = indeterminate;
        markAsDirty();
    }

    @Override
    public void setValue(Boolean newFieldValue)
            throws com.vaadin.data.Property.ReadOnlyException,
            ConversionException {
        super.setValue(newFieldValue);
        getState().checked = newFieldValue;
        markAsDirty();
    }

}
