/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import com.vaadin.ui.Component;
import com.vaadin.ui.CustomComponent;
import com.xmlnamespace.panel.vaadin.shared.ui.MouseWrapperServerRpc;

/**
 * MouseWrapper Widget.
 */
public class MouseWrapper extends CustomComponent {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The listeners. */
    private Set<MouseOverListener> listeners = new CopyOnWriteArraySet<MouseOverListener>();

    /** The rpc. */
    private InnerMouseWrapperServerRpc rpc = new InnerMouseWrapperServerRpc();

    /**
     * Instantiates a new mouse wrapper.
     */
    public MouseWrapper() {
        super();
        registerRpc(rpc);
    }

    /**
     * Instantiates a new mouse wrapper.
     * 
     * @param compositionRoot
     *            the composition root
     */
    public MouseWrapper(Component compositionRoot) {
        super(compositionRoot);
        registerRpc(rpc);
    }

    /**
     * Adds the listener.
     * 
     * @param listener
     *            the listener
     */
    public void addListener(final MouseOverListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes the listener.
     * 
     * @param listener
     *            the listener
     */
    public void removeListener(final MouseOverListener listener) {
        listeners.remove(listener);
    }

    /**
     * The Class MouseOverEvent.
     */
    public static class MouseOverEvent {

        /**
         * The Enum Type.
         */
        public static enum Type {

            /** The MOUS e_ over. */
            MOUSE_OVER,
            /** The MOUS e_ out. */
            MOUSE_OUT
        };

        /** The type. */
        private Type type;

        /**
         * Instantiates a new mouse over event.
         * 
         * @param type
         *            the type
         */
        public MouseOverEvent(final Type type) {
            this.type = type;
        }

        /**
         * Gets the type.
         * 
         * @return the type
         */
        public Type getType() {
            return type;
        }
    }

    /**
     * The listener interface for receiving mouseOver events. The class that is
     * interested in processing a mouseOver event implements this interface, and
     * the object created with that class is registered with a component using
     * the component's <code>addMouseOverListener<code> method. When
     * the mouseOver event occurs, that object's appropriate
     * method is invoked.
     * 
     * @see MouseOverEvent
     */
    public static interface MouseOverListener {

        /**
         * On mouse over.
         * 
         * @param event
         *            the event
         */
        public void onMouseOver(final MouseOverEvent event);
    }

    /**
     * The Class InnerMouseWrapperServerRpc.
     */
    private class InnerMouseWrapperServerRpc implements MouseWrapperServerRpc {

        /** The Constant serialVersionUID. */
        private static final long serialVersionUID = 1L;

        /*
         * (non-Javadoc)
         * 
         * @see
         * com.xmlnamespace.panel.vaadin.shared.ui.MouseWrapperServerRpc#onMouseOver
         * ()
         */
        @Override
        public void onMouseOver() {
            final MouseOverEvent event = new MouseOverEvent(
                    MouseOverEvent.Type.MOUSE_OVER);

            for (final MouseOverListener listener : listeners) {
                listener.onMouseOver(event);
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see
         * com.xmlnamespace.panel.vaadin.shared.ui.MouseWrapperServerRpc#onMouseOut
         * ()
         */
        @Override
        public void onMouseOut() {
            final MouseOverEvent event = new MouseOverEvent(
                    MouseOverEvent.Type.MOUSE_OUT);

            for (final MouseOverListener listener : listeners) {
                listener.onMouseOver(event);
            }
        }

    }

}
