/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import java.util.Set;

import com.vaadin.event.ShortcutAction.KeyCode;
import com.vaadin.ui.TextArea;
import com.xmlnamespace.panel.vaadin.shared.ui.ShortcutBlockingTextAreaState;

/**
 * Text area that blocks key events from propagating out of the text area.
 * 
 * Exceptions can be made by adding propagation keys.
 */
public class ShortcutBlockingTextArea extends TextArea {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;


    /* (non-Javadoc)
     * @see com.vaadin.ui.TextArea#getState()
     */
    @Override
    protected ShortcutBlockingTextAreaState getState() {
        return (ShortcutBlockingTextAreaState) super.getState();
    }

    /**
     * Adds a propagation key.
     * 
     * Propagation keys will not be blocked
     * 
     * @param keyCode
     *            the key code
     */
    public void addPropagationKey(int keyCode) {
        getState().allowedKeyCodes.add(keyCode);
        markAsDirty();
    }

    /**
     * Removes a propagation key.
     * 
     * @param keyCode
     *            the key code
     */
    public void removePropagationKey(KeyCode keyCode) {
        getState().allowedKeyCodes.remove(keyCode);
        markAsDirty();
    }

    /**
     * Gets the propagation keys.
     * 
     * @return the propagation keys
     */
    public Set<Integer> getPropagationKeys() {
        return getState().allowedKeyCodes;
    }

}
